@@ -21,6 +21,7 @@ Most notably:
     
     There are other functions here as well, but these are the most useful/novel'''
 
+from copy import deepcopy
 import operator
 
 ###############################
@@ -84,6 +85,15 @@ def roll(l,n=1):
     else:
         return l
 
+def zipIntoPairs(l,cycle=False,offset=1):
+    '''Form all adjacent pairs from a list
+       If cycle is True, pair the end and beginning as well
+       If offset is greater than 1, pair separated elements'''
+    if cycle:
+        return zip(l,roll(l,-offset))
+    else:
+        return zip(l[:-offset],l[offset:])
+
 def groupByFunction(l,f,appendFun=None):
     '''Break up a list into groups (a dict of smaller lists) based on a
        common property (the result of the function f applied to each element).
@@ -108,6 +118,72 @@ def groupByFunction(l,f,appendFun=None):
         groupDict.setdefault(f(i),[]).append(appendFun(i) if appendFun else i)
     return groupDict
 
+def getElementConnections(connectionSets):
+    '''Take a list of connections and return a dictionary of connections to each element.
+       Examples:
+       >>> getElementConnections([[1,2],[4,5],[2,6]]):
+       {1:[2], 2:[1,6], 4:[5], 5:[4], 6:[2]}
+       >>> getElementConnections([[1,2],[2,3,4]]):
+       {1:[2], 2:[1,3,4], 3:[2,4], 4:[2,3]}
+       
+       connectionSets must be a nested structure with a depth of 2 (i.e. a list-of-lists)
+       and all elements must be immutables (for use in a set)'''
+    # Algorithm summary:
+    # Form a master list of elements (and make them keys to the dictionary)
+    # For any given element, get the connection sets that contain the element
+    #   and distill down unique connections from these
+    return { k:sorted( set(flatten( i for i in connectionSets
+                                      if k in i )).difference([k]) )
+            for k in sorted(set(flatten(connectionSets))) }
+
+    # Equivalent but more complex version
+    # Speed is similar; better under some circumstances, worse under others
+    ##keys = sorted(set(flatten(connectionSets)))
+    ##d = { k:set() for k in keys }
+    ##for c in connectionSets:
+    ##    cs = sorted(set(c))
+    ##    for e in cs:
+    ##        d[e].update(deletecases(cs,e))
+    ##for k,v in d.iteritems():
+    ##    d[k] = sorted(v)
+    ##return d
+
+def _firstOrOther(l,other=None):
+    '''Helper function for getChainsFromConnections
+       Returns the first element if it exists and if not,
+         returns "other" (default None)'''
+    return l[0] if len(l) else other
+
+def getChainsFromConnections(connections,checkConnections=True):
+    '''Take a list of connections and return a list of connection chains
+       connections is a dictionary of connections between elements (which must be hashable)
+         and can be generated using getElementConnections
+       The checkConnections option tests that there is only one one path
+         through each point (aka 2 or fewer connections, no branching)
+       Returns a list of chains (lists of elements)
+       '''
+    connections = deepcopy(connections) # Protect the input from modification
+    if checkConnections: # Check that there is no branching
+        assert all( len(v)<3 for k,v in connections.iteritems() ), 'Aborting; this network has branching'
+    
+    chains = []
+    while len(connections): # loop over possible chains
+        # Pick a starting point (an end point if possible)
+        currPt = _firstOrOther( [ pt for pt,conn in connections.iteritems()
+                                     if len(conn)==1 ],
+                                connections.keys()[0] )
+        # Form a chain and move the current point forward
+        chain = [currPt]
+        currPt = connections.pop(currPt)[0]
+        while currPt: # loop to fill a chain, stop on an invalid
+            chain.append(currPt)
+            if len(connections)==0:
+                break
+            connections[currPt] = deletecases(connections[currPt], [chain[-2]])
+            currPt = _firstOrOther(connections.pop(currPt,[]))
+        chains.append(chain)
+    return chains
+
 def interp(l,index):
     '''Basically floating point indexing with interpolation in between.'''
     m = index % 1
